@@ -1,5 +1,5 @@
-from typing import List, Optional
-from pydantic import BaseModel, Field
+from typing import List
+from pydantic import BaseModel
 
 
 class ContextualPrecisionVerdict(BaseModel):
@@ -11,5 +11,5 @@ class Verdicts(BaseModel):
     verdicts: List[ContextualPrecisionVerdict]
 
 
-class Reason(BaseModel):
+class ContextualPrecisionScoreReason(BaseModel):
     reason: str
