@@ -1,6 +1,7 @@
 from deepeval.openai_agents.callback_handler import DeepEvalTracingProcessor
-from deepeval.openai_agents.runner import Runner
-from deepeval.openai_agents.patch import function_tool
 from deepeval.openai_agents.agent import DeepEvalAgent as Agent
+from deepeval.openai_agents.patch import function_tool
 
-__all__ = ["DeepEvalTracingProcessor", "Runner", "function_tool", "Agent"]
+# from deepeval.openai_agents.runner import Runner
+
+__all__ = ["DeepEvalTracingProcessor", "Agent", "function_tool"]
