@@ -1,4 +1,5 @@
 from pydantic import (
+    ConfigDict,
     Field,
     BaseModel,
     model_validator,
@@ -121,7 +122,9 @@ class ToolCall(BaseModel):
 
         # Handle nested fields like input_parameters
         if self.input_parameters:
-            formatted_input = json.dumps(self.input_parameters, indent=4)
+            formatted_input = json.dumps(
+                self.input_parameters, indent=4, ensure_ascii=False
+            )
             formatted_input = self._indent_nested_field(
                 "input_parameters", formatted_input
             )
@@ -129,7 +132,9 @@ class ToolCall(BaseModel):
 
         # Handle nested fields like output
         if isinstance(self.output, dict):
-            formatted_output = json.dumps(self.output, indent=4)
+            formatted_output = json.dumps(
+                self.output, indent=4, ensure_ascii=False
+            )
             formatted_output = self._indent_nested_field(
                 "output", formatted_output
             )
@@ -151,6 +156,8 @@ class ToolCall(BaseModel):
 
 
 class LLMTestCase(BaseModel):
+    model_config = ConfigDict(extra="ignore")
+
     input: str
     actual_output: Optional[str] = Field(
         default=None,
